@@ -33,11 +33,12 @@ This portion is adding to the :ref:`application events` provided by :class:`~ae.
 events redirected from the Kivy :class:`~kivy.app.App` class (the original Kivy event/callback-method name is
 given in brackets):
 
-* on_kivy_app_build (build).
-* on_kivy_app_start (on_start)
-* on_app_pause (on_pause)
-* on_app_resume (on_resume)
-* on_kivy_app_stop (on_stop)
+* on_app_build (after main_app.on_app_run and before kivy.app.App.build).
+* on_app_built (after kivy.app.App.build).
+* on_app_started (kivy.app.App.on_start)
+* on_app_pause (kivy.app.App.on_pause)
+* on_app_resume (kivy.app.App.on_resume)
+* on_app_stopped (kivy.app.App.on_stop)
 
 
 kivy widget classes
@@ -46,7 +47,7 @@ kivy widget classes
 The widgets provided by this portion are based on the kivy widgets and are respecting the :ref:`app-state-variables`
 specifying the desired app style (dark or light) and font size.
 
-Most of them also change automatically the :ref:`application-flow`.
+Most of them also change automatically the :ref:`application flow`.
 
 The following widgets provided by this portion will be registered in the kivy widget class maps by importing this module
 for to be available for your app:
@@ -104,9 +105,9 @@ import kivy.uix.textinput
 from kivy.uix.textinput import TextInput, TextInputCutCopyPaste as _TextInputCutCopyPaste   # type: ignore
 from kivy.uix.widget import Widget                                                          # type: ignore
 
-from ae.base import sys_platform                                                            # type: ignore
+from ae.base import os_platform                                                             # type: ignore
 from ae.paths import app_docs_path                                                          # type: ignore
-from ae.files import FilesRegister, CachedFile                                              # type: ignore
+from ae.files import CachedFile                                                             # type: ignore
 from ae.i18n import default_language, get_f_string, get_text                                # type: ignore
 from ae.core import DEBUG_LEVEL_DISABLED, DEBUG_LEVEL_ENABLED                               # type: ignore
 
@@ -123,19 +124,13 @@ from ae.kivy_help import HelpBehavior, HelpLayout, HelpToggler
 from ae.kivy_relief_canvas import ReliefCanvas                                              # type: ignore
 
 
-__version__ = '0.1.58'
+__version__ = '0.1.66'
 
 
 kivy.require('2.0.0')
 # 1.9.1 is needed for Window.softinput_mode 'below_target'
 # 2.0.0 is needed for Animation Sequence (>= 2.0.0rc2) and ScrollView recursion (> 2.0.0rc3) bug fixes
 
-# if the entry field is on top of the screen then it will be disappear with below_target mode
-# and in the default mode ('') the keyboard will cover the entry field if it is in the lower part of the screen
-# therefore commented out the following two code lines (and setting it now depending on the entry field y position)
-# if Window:                                  # is None on gitlab ci
-#    Window.softinput_mode = 'below_target'   # ensure android keyboard is not covering Popup/text input if at bottom
-
 MAIN_KV_FILE_NAME = 'main.kv'   #: default file name of the main kv file
 
 """ sine 3 x deeper repeating animation, used e.g. for to animate ae.kivy_help.HelpLayout
@@ -311,7 +306,7 @@ Builder.load_string('''\
     help_id: app.main_app.help_flow_id(self.tap_flow_id)
     help_vars: dict(new_flow_id=self.tap_flow_id, self=self)
     icon_name: ""
-    on_release: app.main_app.change_flow(self.tap_flow_id, **self.tap_kwargs)
+    on_state: app.main_app.change_flow(self.tap_flow_id, **self.tap_kwargs)
     source:
         app.main_app.img_file(self.icon_name or flow_key_split(self.tap_flow_id)[0], \
                               app.app_states['font_size'], app.app_states['light_theme'])
@@ -364,6 +359,24 @@ def ensure_tap_kwargs_refs(init_kwargs: Dict[str, Any], tap_widget: Widget):
     popup_kwargs['parent'] = popup_kwargs.get('parent', tap_widget)
 
 
+def update_event_kwargs(widget: Widget, popup_kwargs: Optional[Dict[str, Any]] = None, **tap_kwargs) -> Dict[str, Any]:
+    """ update widget's tap_kwargs property and return the updated dictionary (for kv rule of tap_kwargs).
+
+    :param widget:              widget with tap_kwargs property to be updated.
+    :param popup_kwargs:        dict with items for to update popup_kwargs key of tap_kwargs
+    :param tap_kwargs:          additional tap_kwargs items to update.
+    :return:
+    """
+    handle = dict(tap_kwargs=widget.tap_kwargs)
+    ensure_tap_kwargs_refs(handle, widget)
+    new_kwargs = handle['tap_kwargs']
+    if popup_kwargs:
+        new_kwargs['popup_kwargs'].update(popup_kwargs)
+    if tap_kwargs:
+        new_kwargs.update(tap_kwargs)
+    return new_kwargs
+
+
 class AppStateSlider(HelpBehavior, Slider):
     """ slider widget with help text for to change app state value. """
     app_state_name = StringProperty()   #: name of the app state to be changed by this slider value
@@ -371,7 +384,7 @@ class AppStateSlider(HelpBehavior, Slider):
 
 class ImageLabel(ReliefCanvas, Label):
     """ base label used for all labels and buttons. """
-    _touch_anim = NumericProperty(1.0)
+    _touch_anim = NumericProperty(1.0)  #: used for animation to display that the widget got touched
 
 
 class ImageButton(ButtonBehavior, ImageLabel):                                               # pragma: no cover
@@ -615,12 +628,13 @@ class FlowInput(HelpBehavior, TextInput):
                                 Set index to zero if the old/last index was on the last item in the matching list.
         """
         cnt = len(self._matching_ac_texts)
-        chi = list(reversed(self._ac_dropdown.container.children))
-        idx = self._matching_ac_index
-        chi[idx].square_fill_ink = Window.clearcolor
-        self._matching_ac_index = (idx + delta + cnt) % cnt
-        chi[self._matching_ac_index].square_fill_ink = self.auto_complete_selector_index_ink
-        self.suggestion_text = self._matching_ac_texts[self._matching_ac_index][len(self.text):]    # type: ignore #mypy
+        if cnt:
+            chi = list(reversed(self._ac_dropdown.container.children))
+            idx = self._matching_ac_index
+            chi[idx].square_fill_ink = Window.clearcolor
+            self._matching_ac_index = (idx + delta + cnt) % cnt
+            chi[self._matching_ac_index].square_fill_ink = self.auto_complete_selector_index_ink
+            self.suggestion_text = self._matching_ac_texts[self._matching_ac_index][len(self.text):]  # type: ignore
 
     def _delete_ac_text(self, ac_text: str = ""):
         if not ac_text and self._matching_ac_texts:
@@ -657,7 +671,7 @@ class FlowInput(HelpBehavior, TextInput):
         """
         key_name = keycode[1]
         if self._ac_dropdown.attach_to:
-            if key_name in ('enter', 'right'):
+            if key_name in ('enter', 'right') and len(self._matching_ac_texts) > self._matching_ac_index:
                 self.suggestion_text = ""
                 self.text = self._matching_ac_texts[self._matching_ac_index]
                 self._ac_dropdown.dismiss()
@@ -813,7 +827,8 @@ class FrameworkApp(App):
 
         :return:                root widget (Main instance) of this app.
         """
-        self.main_app.call_method('on_kivy_app_build')
+        self.main_app.vpo("FrameworkApp.build")
+        self.main_app.call_method('on_app_build')
         Window.bind(on_resize=self.win_pos_size_change,
                     left=self.win_pos_size_change,
                     top=self.win_pos_size_change,
@@ -821,6 +836,7 @@ class FrameworkApp(App):
                     on_key_up=self.key_release_from_kivy)
 
         self.main_app.framework_root = root = Factory.Main()
+        self.main_app.call_method('on_app_built')
         return root
 
     def key_press_from_kivy(self, keyboard: Any, key_code: int, _scan_code: int, key_text: Optional[str],
@@ -845,17 +861,39 @@ class FrameworkApp(App):
         """
         return self.main_app.call_method('on_key_release', keyboard.command_keys.get(key_code, str(key_code)))
 
-    def on_start(self):
-        """ kivy app start event, called after :meth:`MainAppBase.run_app` method and MainAppBase.on_app_start event.
+    def on_file_chooser_entry_added(self, view_entries: List[Widget]):                                # pragma: no cover
+        """ on_entry_added/on_subentry_to_entry event handler for to patch theme-related properties of FileChooser.
 
-        Kivy just created the main layout by calling its :meth:`~kivy.app.App.build` method and
-        attached it to the main window.
+        :param view_entries:    list of view entries for a node (icon or label) of the file chooser.
 
-        Emits the `on_kivy_app_start` event.
-       """
-        self.main_app.framework_win = self.root.parent
-        self.win_pos_size_change()  # init. app./self.landscape (on app startup and after build)
-        self.main_app.call_method('on_kivy_app_start')
+        .. note::
+            This method get called for each node in the moment when a file entry widget (FileListEntry or FileIconEntry)
+            gets added to the file chooser (FileChooser widget).
+
+            Therefore the patches done here are not affected if the user preferences (e.g. the font size or light/dark
+            theme) get changed while a file chooser instance is displayed. In this case the user has to simply close
+            and reopen/re-instantiate the file chooser for to display the nodes with the just changed user preferences.
+
+            Theme adaption is still missing for the file chooser progress: all font sizes and colors of the currently
+            used :class:`~kivy.uix.filechooser.FileChooserProgress` are hard-coded, so a theme-aware progress class
+            has to be implemented (and assigned to the :attr:`~kivy.uix.filechooser.FileChooserController.progress_cls`
+            property).
+
+        """
+        for entry in view_entries:
+            if 'FileListEntry' in str(entry):               # isinstance(entry, Factory.FileListEntry) -> TypeError
+                box, entry = entry, entry.children[0]       # children[0] is BoxLayout of FileListEntry
+                # box.children[0].children[1] is box.ids.filename
+                box.color_selected = [0.69, 0.69, 0.69, 1.0] if self.app_states['light_theme'] else [0.3, 0.3, 0.3, 1.0]
+                entry.children[1].color = self.font_color   # children[1] is file name label
+                entry.children[1].font_size = min(box.height * 0.90, self.app_states['font_size'])
+                entry.children[0].color = self.font_color   # children[0] is file size label
+                entry.children[0].font_size = min(box.height * 0.69, self.app_states['font_size'])
+            elif 'FileIconEntry' in str(entry):             # TypeError if using isinstance()
+                entry.children[1].color = self.font_color
+                entry.children[1].font_size = min(entry.children[1].height * 0.99, self.app_states['font_size'])
+                entry.children[0].color = self.font_color if self.app_states['light_theme'] else [0.81, 0.81, 0.81, 1.0]
+                entry.children[0].font_size = min(entry.children[0].height * 0.90, self.app_states['font_size'])
 
     def on_pause(self) -> bool:
         """ app pause event automatically saving the app states.
@@ -864,6 +902,7 @@ class FrameworkApp(App):
 
         :return:                True.
         """
+        self.main_app.vpo("FrameworkApp.on_pause")
         self.main_app.save_app_states()
         self.main_app.call_method('on_app_pause')
         return True
@@ -875,18 +914,33 @@ class FrameworkApp(App):
 
         :return:                True.
         """
+        self.main_app.vpo("FrameworkApp.on_resume")
         self.main_app.load_app_states()
         self.main_app.call_method('on_app_resume')
         return True
 
+    def on_start(self):
+        """ kivy app start event, called after :meth:`MainAppBase.run_app` method and MainAppBase.on_app_start event.
+
+        Kivy just created the main layout by calling its :meth:`~kivy.app.App.build` method and
+        attached it to the main window.
+
+        Emits the `on_app_started` event.
+       """
+        self.main_app.vpo("FrameworkApp.on_start")
+        self.main_app.framework_win = self.root.parent
+        self.win_pos_size_change()  # init. app./self.landscape (on app startup and after build)
+        self.main_app.call_method('on_app_started')
+
     def on_stop(self):
         """ quit app event automatically saving the app states.
 
-        Emits the `on_kivy_app_stop` event whereas the method :meth:`~ae.gui_app.MainAppBase.stop_app`
+        Emits the `on_app_stopped` event whereas the method :meth:`~ae.gui_app.MainAppBase.stop_app`
         emits the `on_app_stop` event.
         """
+        self.main_app.vpo("FrameworkApp.on_stop")
         self.main_app.save_app_states()
-        self.main_app.call_method('on_kivy_app_stop')
+        self.main_app.call_method('on_app_stopped')
 
     def win_pos_size_change(self, *_):
         """ resize handler updates: :attr:`~ae.gui_app.MainAppBase.win_rectangle`, :attr:`~FrameworkApp.landscape`. """
@@ -1013,7 +1067,7 @@ class KivyMainApp(HelpAppBase):
     unselected_item_ink: tuple = (0.39, 0.39, 0.39, 0.18)   #: rgba color for list items (unselected)
 
     get_txt_ = get_txt                                      #: make i18n translations available via main app instance
-    kbd_input_mode: str = 'pan'                             #: optional app state for to set Window[Base].softinput_mode
+    kbd_input_mode: str = 'scale'                           #: optional app state for to set Window[Base].softinput_mode
     documents_root_path: str = "."                          #: root file path for app documents, e.g. for import/export
 
     _debug_enable_clicks: int = 0
@@ -1100,8 +1154,8 @@ class KivyMainApp(HelpAppBase):
 
     def load_sounds(self):
         """ override for to pre-load audio sounds from app folder snd into sound file cache. """
-        self.sound_files = FilesRegister('snd/**', file_class=CachedFile,
-                                         object_loader=lambda f: SoundLoader.load(f.path))
+        super().load_sounds()   # load from sound file paths all files into :class:`~ae.files.RegisteredFile` instances
+        self.sound_files.reclassify(object_loader=lambda f: SoundLoader.load(f.path))   # :class:`~ae.files.CachedFile`
 
     def mix_background_ink(self):
         """ remix background ink if one of the basic back colours change. """
@@ -1118,11 +1172,12 @@ class KivyMainApp(HelpAppBase):
 
     def on_app_start(self):                                                                     # pragma: no cover
         """ app start event handler - used for to set the window pos and size. """
-        # super().on_app_start()      # call of ae.gui_app.MainAppBase.on_app_start() not needed
+        super().on_app_start()      # ae.gui_app.MainAppBase.on_app_start() does the i18n initialization
         get_txt.switch_lang(self.lang_code)
         self.change_light_theme(self.light_theme)
+        Window.softinput_mode = self.kbd_input_mode
 
-        if sys_platform() not in ('android', 'ios'):    # ignore last win pos on android/iOS, use always the full screen
+        if os_platform not in ('android', 'ios'):       # ignore last win pos on android/iOS, use always the full screen
             win_rect = self.win_rectangle
             if win_rect:                                # is empty tuple at very first app start
                 Window.left, Window.top = win_rect[:2]
@@ -1146,6 +1201,7 @@ class KivyMainApp(HelpAppBase):
         self.vpo(f"MainAppBase.on_kbd_input_mode_change to {mode}")
         self.change_app_state('kbd_input_mode', mode)
         self.set_var('kbd_input_mode', mode, section=APP_STATE_SECTION_NAME)  # add optional app state var to config
+        Window.softinput_mode = mode
         return True
 
     def on_lang_code(self):
@@ -1211,21 +1267,6 @@ class KivyMainApp(HelpAppBase):
             except Exception as ex:
                 self.po(f"KivyMainApp.play_vibrate exception {ex}")
 
-    def prevent_keyboard_covering(self, input_box_bottom: float) -> bool:
-        """ prevent that the virtual keyboard popping up on mobile platforms is covering the text input field.
-
-        :param input_box_bottom:    y position of the bottom of the input field box.
-        :return:                    True if keyboard is covering the passed y/bottom position, else False.
-        """
-        if sys_platform() != 'android':
-            return False
-
-        keyboard_height = Window.keyboard_height or Window.height / 2  # 'or'-fallback because SDL2 reports 0 kbd height
-        mode_changed = input_box_bottom < keyboard_height
-        Window.softinput_mode = self.kbd_input_mode if mode_changed else ''
-
-        return mode_changed
-
     def setup_app_states(self, app_states: AppStateType):
         """ put app state variables into main app instance for to prepare framework app.run_app.
 
@@ -1268,12 +1309,6 @@ class KivyMainApp(HelpAppBase):
         # framework_win has absolute screen coordinates and lacks x, y properties, therefore use app.root as def parent
         parent = popup_kwargs.pop('parent', self.framework_root)
         popup_instance = popup_class(**popup_kwargs)
-        if self.prevent_keyboard_covering(popup_instance.y):
-            container = getattr(popup_instance, '_container', None)
-            if container:
-                container.clear_widgets()                       # clear container for Popup only
-            popup_instance = popup_class(**popup_kwargs)        # new instance if kbd covering popup
-
         popup_instance.open(parent)
 
         return popup_instance
